<?php
/**
 * Retrieve form entry data for use in charts.
 *
 * @package GravityKit\GravityCharts
 */

namespace GravityKit\GravityCharts;

use Exception;
use GF_Field;
use GF_Query;
use GF_Query_Condition;
use GFAPI;
use GFCommon;
use GFFormsModel;
use GravityKit\GravityCharts\QueryFilters\QueryFilters;
use WP_REST_Request;
use WP_REST_Server;

/**
 * GravityCharts API.
 */
class API {
	/**
	 * REST API Namespace.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	const NAMESPACE = 'gravitycharts/v1';

	/**
	 * GravityCharts plugin instance.
	 *
	 * @since 1.0
	 *
	 * @var Plugin
	 */
	private $gravitycharts;

	/**
	 * Class instance.
	 *
	 * @since 1.0.4
	 *
	 * @var API
	 */
	private static $_instance;

	/**
	 * REST API constructor.
	 *
	 * @since 1.0
	 */
	public function __construct() {

		$this->gravitycharts = Plugin::get_instance();

		add_action( 'rest_api_init', [ $this, 'init' ] );
	}

	/**
	 * Returns class instance.
	 *
	 * @since 1.0.4
	 *
	 * @return API
	 */
	public static function get_instance(): API {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new API();
		}

		return self::$_instance;
	}

	/**
	 * Initializes API.
	 *
	 * @since 1.0
	 */
	public function init() {

		$this->gravitycharts->init();

		register_rest_route(
			self::NAMESPACE,
			'/chart',
			[
				'methods'             => WP_REST_Server::READABLE,
				'args'                => [
					'feed_id' => [
						'required' => true,
						'type'     => 'number',
					],
				],
				'permission_callback' => [ $this, 'permissions' ],
				'callback'            => function ( WP_REST_Request $request ) {
					$feed_id  = intval( $request->get_param( 'feed_id' ) );
					$response = $this->get_chart( $feed_id );

					return rest_ensure_response( $response );
				},
			]
		);

		register_rest_route(
			self::NAMESPACE,
			'/chart-options',
			[
				'methods'             => WP_REST_Server::READABLE,
				'args'                => [
					'feed_id' => [
						'required' => true,
						'type'     => 'number',
					],
				],
				'permission_callback' => [ $this, 'permissions' ],
				'callback'            => function ( WP_REST_Request $request ) {
					$feed_id  = intval( $request->get_param( 'feed_id' ) );
					$response = $this->get_chart_options( $feed_id );

					return rest_ensure_response( $response );
				},
			]
		);

		register_rest_route(
			self::NAMESPACE,
			'/chart-data',
			[
				'methods'             => WP_REST_Server::CREATABLE,
				'args'                => [
					'form_id'   => [
						'required' => true,
						'type'     => 'number',
					],
					'field_id'  => [
						'required' => true,
						'type'     => 'number',
					],
					'condition' => [
						'required' => false,
						'type'     => 'string',
					],
				],
				'permission_callback' => [ $this, 'permissions' ],
				'callback'            => function ( WP_REST_Request $request ) {
					$form_id           = intval( $request->get_param( 'form_id' ) );
					$feed_id           = intval( $request->get_param( 'feed_id' ) );
					$field_id          = intval( $request->get_param( 'field_id' ) );
					$filters           = $request->get_param( 'filters' );
					$conditional_logic = 'null';

					if ( $filters && 'null' !== $filters ) {
						try {
							$conditional_logic = json_decode( stripslashes( $filters ), true );
						} catch ( Exception $e ) {
							$conditional_logic = 'null';
						}
					}

					$response = $this->get_chart_data( $form_id, $feed_id, $field_id, $conditional_logic );

					return rest_ensure_response( $response );
				},
			]
		);

		register_rest_route(
			self::NAMESPACE,
			'/forms',
			[
				'methods'             => WP_REST_Server::READABLE,
				'permission_callback' => [ $this, 'permissions' ],
				'callback'            => function () {
					$response = $this->get_forms();

					return rest_ensure_response( $response );
				},
			]
		);

		register_rest_route(
			self::NAMESPACE,
			'/feeds',
			[
				'methods'             => WP_REST_Server::READABLE,
				'args'                => [
					'form_id' => [
						'required' => true,
						'type'     => 'number',
					],
				],
				'permission_callback' => [ $this, 'permissions' ],
				'callback'            => function ( WP_REST_Request $request ) {
					$form_id  = intval( $request->get_param( 'form_id' ) );
					$response = $this->get_feeds( $form_id );

					return rest_ensure_response( $response );
				},
			]
		);
	}

	/**
	 * Checks REST API permissions.
	 *
	 * @since 1.0
	 *
	 * @return boolean
	 */
	public function permissions(): bool {
		$capabilities = $this->gravitycharts->chart_feed->get_capabilities( 'form_settings' );

		return GFCommon::current_user_can_any( $capabilities );
	}

	/**
	 * Gets chart type, options, and data.
	 *
	 * @since 1.0
	 *
	 * @param integer $feed_id The feed ID.
	 *
	 * @return array
	 */
	public function get_chart( int $feed_id ): array {
		$response = [];
		$feed     = GFAPI::get_feed( $feed_id );

		if ( is_wp_error( $feed ) ) {
			Plugin::get_instance()->chart_feed->log_error( 'Error fetching feed for feed ID ' . esc_html( $feed_id ) . ': ' . $feed->get_error_message() );
			return [];
		}

		if (
			! isset( $feed['meta'] ) ||
			! isset( $feed['meta']['dataField'] ) ||
			! isset( $feed['meta']['chartType'] ) ||
			! isset( $feed['form_id'] )
		) {
			return $response;
		}

		$filters = rgars( $feed, 'meta/conditional_logic', 'null' );

		$response['type']      = $feed['meta']['chartType'];
		$response['options']   = $this->get_chart_options( $feed_id );
		$response['data']      = $this->get_chart_data( $feed['form_id'], $feed_id, $feed['meta']['dataField'], $filters );
		$response['ariaLabel'] = $this->get_chart_description( $response['data'], $response['options'], $response['type'] );

		return $response;
	}

	/**
	 * Generates an assistive text description for the chart.
	 *
	 * @param array  $data Chart data, as fetched by {@see get_chart_data}.
	 * @param array  $options Chart options from {@see get_chart_options}.
	 * @param string $type Slug of the type of chart ("bar", "pie", "radarArea", etc.).
	 *
	 * @return string
	 */
	public function get_chart_description( array $data, array $options, string $type ): string {

		$data_description = [];

		// TODO: When we have multiple datasets the labels array will need to be nested.
		foreach ( $data['datasets'] as $dataset_key => $dataset ) {

			$values = array_combine( array_values( $data['labels'] ), array_values( $dataset['data'] ) );

			// translators: The placeholder is replaced with the number of the dataset being described in the chart data.
			$dataset_output = sprintf( esc_html__( 'Dataset %d: ', 'gk-gravitycharts' ), $dataset_key + 1 );

			$glue = '; ';
			foreach ( $values as $key => $value ) {
				$dataset_output .= sprintf( '%s: %s', esc_html( $key ), esc_html( $value ) . $glue );
			}

			$dataset_output = rtrim( $dataset_output, $glue );

			$data_description[] = $dataset_output;
		}

		switch ( $type ) {
			case 'pie':
				// translators: Do not translate the words inside the {} curly brackets; they are replaced.
				$description_template = esc_html__( 'A {chart type} chart with the following data: {data description}.', 'gk-gravitycharts' );
				break;
			default:
				// translators: Do not translate the words inside the {} curly brackets; they are replaced.
				$description_template = esc_html__( 'A {chart type} chart, values from {min} to {max}, with the following data: {data description}.', 'gk-gravitycharts' );
				break;
		}

		$chart_type       = Chart_Types::get( $type );
		$chart_type_label = rgar( $chart_type, 'label', $type );

		return strtr(
			$description_template,
			[
				'{chart type}'       => mb_strtolower( $chart_type_label ),
				'{min}'              => rgars( $options, 'scales/y/min', 0 ),
				'{max}'              => rgars( $options, 'scales/y/max', esc_html__( 'the maximum value', 'gk-gravitycharts' ) ),
				'{data description}' => implode( '. ', $data_description ),
			]
		);
	}

	/**
	 * Gets chart options.
	 *
	 * @since 1.0
	 *
	 * @param integer $feed_id The feed ID.
	 *
	 * @return array
	 */
	protected function get_chart_options( int $feed_id ): array {
		$response = [];
		$feed     = GFAPI::get_feed( $feed_id );

		if ( empty( $feed ) ) {
			Plugin::get_instance()->chart_feed->log_error( 'Feed #' . esc_html( $feed_id ) . ' not found.' );
			return $response;
		}

		if ( is_wp_error( $feed ) ) {
			Plugin::get_instance()->chart_feed->log_error( 'Error fetching feed #' . esc_html( $feed_id ) . ': ' . $feed->get_error_message() );
			return $response;
		}

		if ( ! isset( $feed['meta'] ) || ! isset( $feed['meta']['chartType'] ) ) {
			return $response;
		}

		$type = $feed['meta']['chartType'];

		if ( 'pie' === $type || 'polarArea' === $type ) {
			unset( $feed['meta']['backgroundColor'] );
			unset( $feed['meta']['borderColor'] );
		} else {
			unset( $feed['meta']['backgroundColors'] );
			unset( $feed['meta']['borderColors'] );
		}

		$this->gravitycharts->init();

		$response    = $this->gravitycharts->chart_feed->default_chartjs_options();
		$options_map = $this->gravitycharts->chart_feed->settings_to_chartjs_options_map();

		foreach ( $options_map as $option_path => $meta_keys ) {
			foreach ( $meta_keys as $meta_key ) {
				if ( isset( $feed['meta'][ $meta_key ] ) ) {
					$this->set_dot_notation_value(
						$response,
						$option_path,
						$feed['meta'][ $meta_key ]
					);
				}
			}
		}

		$auto_scale = $feed['meta']['autoScale'] ?? false;
		$max        = (int) rgars( $feed, 'meta/max', 1 );
		$min        = (int) rgars( $feed, 'meta/min', 0 );

		if ( 'line' === $type || ( 'bar' === $type && 'x' === $response['indexAxis'] ) ) {
			if ( ! $auto_scale ) {
				$response['scales']['y']['max'] = $max;
				$response['scales']['y']['min'] = $min;
			}

			unset( $response['scales']['r'] );
			unset( $response['scales']['x'] );
		}
		if ( 'bar' === $type && 'y' === $response['indexAxis'] ) {
			if ( ! $auto_scale ) {
				$response['scales']['x']['max'] = $max;
				$response['scales']['x']['min'] = $min;
			}

			unset( $response['scales']['r'] );
			unset( $response['scales']['y'] );
		}
		if ( 'radar' === $type || 'polarArea' === $type ) {
			if ( ! $auto_scale ) {
				$response['scales']['r']['max'] = $max;
				$response['scales']['r']['min'] = $min;
			}

			unset( $response['scales']['x'] );
			unset( $response['scales']['y'] );
		}
		if ( 'pie' === $type ) {
			unset( $response['scales'] );
		}

		return $response;
	}

	/**
	 * Gets chart data.
	 *
	 * @since 1.0
	 *
	 * @param integer      $form_id  The form ID.
	 * @param integer      $feed_id  The feed ID.
	 * @param integer      $field_id The field ID.
	 * @param string|array $filters  Any conditional logic filters applied to the chart.
	 *
	 * @return array
	 */
	protected function get_chart_data( int $form_id, int $feed_id, int $field_id, $filters ): array {
		$form  = GFAPI::get_form( $form_id );
		$field = GFAPI::get_field( $form, $field_id );
		$feed  = GFAPI::get_feed( $feed_id );

		if ( empty( $field->choices ) ) {
			return [
				'labels'   => [],
				'datasets' => [],
			];
		}

		if ( is_wp_error( $feed ) ) {
			Plugin::get_instance()->chart_feed->log_error( 'Error fetching feed #' . esc_html( $feed_id ) . ': ' . $feed->get_error_message() );
			$feed = [];
		}

		/**
		 * Modifies the entries object used for chart data.
		 *
		 * @filter `gk/gravitycharts/api/entries`
		 *
		 * @since 1.0
		 *
		 * @param null $entries
		 * @param array $feed
		 * @param array $form
		 */
		$entries = apply_filters( 'gk/gravitycharts/api/entries', null, $feed, $form, $filters );

		if ( is_null( $entries ) && ! empty( $feed ) && 'null' !== $filters ) {
			try {
				$query_filters = new QueryFilters();
				$query_filters->set_form( GFAPI::get_form( $form_id ) );
				$query_filters->set_filters( $filters );

				$conditions = $query_filters->get_query_conditions();

				$query = new GF_Query(
					$form_id,
					[
						'field_filters' => [],
						'status'        => 'active',
					]
				);

				$query_parts = $query->_introspect();

				$query->where( GF_Query_Condition::_and( $query_parts['where'], $conditions ) );

				$entries = $query->get();
			} catch ( Exception $e ) {
				$entries = null;
			}
		}

		if ( is_null( $entries ) ) {
			$paging = [
				'offset'    => 0,
				'page_size' => 0,
			];

			$entries = GFAPI::get_entries(
				$form_id,
				[
					'status' => 'active',
				],
				null,
				$paging
			);
		}

		if ( is_wp_error( $entries ) ) {
			Plugin::get_instance()->chart_feed->log_error( 'Error fetching entries: ' . $entries->get_error_message() );
			return [];
		}

		// Checkboxes use the "inputs" property, not choices.
		if ( isset( $field->inputs ) ) {
			$chart_data = $this->parse_checkbox_entries( $entries, $field );
		} else {
			$chart_data = $this->parse_standard_entries( $entries, $field );
		}

		// Apply colors.
		$color_palettes = Chart_Feed::get_color_palettes();
		$feed_palette   = rgars( $feed, 'meta/palette', 'default' );
		$color_palette  = $color_palettes[ $feed_palette ];
		$border_align   = (bool) rgars( $feed, 'meta/offset', 0 ) ? 'center' : 'inner';

		$chart_data['datasets'][0] = array_merge(
			$chart_data['datasets'][0],
			[
				'backgroundColor'      => $color_palette['backgroundColor'],
				'borderColor'          => $color_palette['borderColor'],
				'pointBackgroundColor' => $color_palette['pointBackgroundColor'],
				'borderAlign'          => $border_align,
			]
		);

		return $chart_data;
	}

	/**
	 * Gets all Gravity Forms that have active Chart feeds.
	 *
	 * @since 1.0
	 *
	 * @return array{value: int, label: string} Forms with feeds or empty array if none found.
	 */
	protected function get_forms(): array {
		global $wpdb;

		$feeds_table = $wpdb->prefix . 'gf_addon_feed';
		$forms_table = GFFormsModel::get_form_table_name();
		$addon_slug  = $this->gravitycharts->chart_feed->get_slug();

		// phpcs:disable WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		return $wpdb->get_results(
			$wpdb->prepare(
				"SELECT `form`.`id` as value, `form`.`title` as label
					FROM $feeds_table feed
					JOIN $forms_table form
					ON `feed`.`form_id` = `form`.`id`
					WHERE 1 = 1
					    AND `form`.`is_trash` = 0
						AND `feed`.`is_active` = 1
						AND `feed`.`addon_slug` = %s
					GROUP BY value
					ORDER BY `form`.`title` ASC",
				[ $addon_slug ]
			),
			ARRAY_A
		);
		// phpcs:enable WordPress.DB.PreparedSQL.InterpolatedNotPrepared
	}

	/**
	 * Gets all GravityCharts feeds for a given form.
	 *
	 * @since 1.0
	 *
	 * @param integer $form_id The form ID.
	 *
	 * @return array{value: int, label: string} Feed IDs and names or empty array if none found.
	 */
	protected function get_feeds( int $form_id ): array {
		$response = [];
		$feeds    = GFAPI::get_feeds( null, $form_id, $this->gravitycharts->chart_feed->get_slug() );

		if ( is_wp_error( $feeds ) ) {
			Plugin::get_instance()->chart_feed->log_error( 'Error fetching feeds for form ID ' . esc_html( $form_id ) . ': ' . $feeds->get_error_message() );
			return [];
		}

		foreach ( $feeds as $feed ) {
			if ( empty( $feed['id'] ) ) {
				continue;
			}
			$response[] = [
				'value' => $feed['id'],
				'label' => $this->gravitycharts->chart_feed->get_column_value_chartName( $feed ),
			];
		}

		return $response;
	}

	/**
	 * Parses checkbox entry values into a format that is ready for use in Chart.js.
	 *
	 * @since 1.0
	 *
	 * @param array    $entries The form entries.
	 * @param GF_Field $field   The field to retrieve the value from.
	 *
	 * @return array
	 */
	protected function parse_standard_entries( array $entries, GF_Field $field ): array {
		$data   = [];
		$labels = [];

		foreach ( $field->choices as $key => $choice ) {
			$text  = $choice['text'];
			$value = $choice['value'];

			$labels[ $key ] = $text;
			$data[ $value ] = 0;
		}

		foreach ( $entries as $entry ) {
			if ( empty( $entry[ $field->id ] ) ) {
				continue;
			}

			// TODO: use $field->get_selected_choice() method when minimum GF version is bumped to ≥2.5.11.
			$choice = [];
			foreach ( $field->choices as $_choice ) {
				if ( GFFormsModel::choice_value_match( $field, $_choice, $entry[ $field->id ] ) ) {
					$choice = $_choice;
				}
			}

			$value = $choice['value'] ?? null;

			$decoded_values = null;
			if ( ! empty( $value ) ) {
				// Sometimes the value is json encoded, so try to decode.
				$decoded_values = json_decode( $value, true );
			}

			if ( is_array( $decoded_values ) ) {
				foreach ( $decoded_values as $decoded_value ) {
					if ( isset( $data[ $decoded_value ] ) ) {
						$data[ $decoded_value ] ++;
					}
				}
			}

			if ( isset( $data[ $value ] ) ) {
				$data[ $value ] ++;
			}
		}

		return [
			'labels'   => $labels,
			'datasets' => [
				[
					'label' => $field['label'],
					'data'  => array_values( $data ),
				],
			],
		];
	}

	/**
	 * Parses checkbox entry values into a format that is ready for use in Chart.js.
	 *
	 * @since 1.0
	 *
	 * @param array    $entries The form entries.
	 * @param GF_Field $field   The field to retrieve the value from.
	 *
	 * @return array
	 */
	protected function parse_checkbox_entries( array $entries, GF_Field $field ): array {
		$data   = [];
		$labels = [];

		foreach ( $field->inputs as $key => $choice ) {
			$text      = $choice['label'];
			$choice_id = $choice['id'];

			$labels[ $key ]     = $text;
			$data[ $choice_id ] = 0;

			foreach ( $entries as $entry ) {
				if ( empty( $entry[ $choice_id ] ) ) {
					continue;
				}

				if ( isset( $data[ $choice_id ] ) ) {
					$data[ $choice_id ] ++;
				}
			}
		}

		return [
			'labels'   => $labels,
			'datasets' => [
				[
					'label' => $field['label'],
					'data'  => array_values( $data ),
				],
			],
		];
	}

	/**
	 * Sets a nested array value based on dot notation.
	 *
	 * @since 1.0
	 *
	 * @param array  $array The array to update.
	 * @param string $path  The dot notation string.
	 * @param mixed  $value The value to set.
	 */
	protected function set_dot_notation_value( array &$array, string $path, $value ) {
		$location = &$array;
		$steps    = explode( '.', $path );
		foreach ( $steps as $step ) {
			$location = &$location[ $step ];
		}

		$location = $value;
	}
}
