<?php
/**
 * Gravity Forms Chart Feed Add-on.
 *
 * @package GravityKit\GravityCharts
 */

namespace GravityKit\GravityCharts;

use Closure;
use Exception;
use GFForms;
use GFFeedAddOn;
use GravityKitFoundation;
use Gravity_Forms\Gravity_Forms\Settings\Fields;
use Gravity_Forms\Gravity_Forms\Settings\Fields\Base;
use GravityKit\GravityCharts\QueryFilters\QueryFilters;

GFForms::include_feed_addon_framework();

/**
 * GravityCharts Chart Feed.
 */
class Chart_Feed extends GFFeedAddOn {
	/**
	 * Minimum Gravity Forms version.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	protected $_min_gravityforms_version = '2.5';

	/**
	 * GravityCharts slug.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	protected $_slug = 'gravitycharts';

	/**
	 * GravityCharts title.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	protected $_title = 'GravityCharts Feed Add-On';

	/**
	 * GravityCharts short title.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	protected $_short_title = 'GravityCharts';

	/**
	 * The plugin basename.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	protected $_path;

	/**
	 * The full path to the plugin.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	protected $_full_path;

	/**
	 * The version of the plugin.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	protected $_version = GK_GRAVITYCHARTS_PLUGIN_VERSION;

	/**
	 * The capability required to access the plugin.
	 *
	 * @since 1.0.2
	 *
	 * @var string
	 */
	protected $_capabilities_form_settings = 'manage_options';

	/**
	 * Query Filters instance.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	private $query_filters;

	/**
	 * Class instance.
	 *
	 * @since 1.0.4
	 *
	 * @var Chart_Feed
	 */
	private static $_instance;

	/**
	 * {@inheritdoc}
	 *
	 * @since 1.0
	 */
	public function __construct() {
		$this->_path         = plugin_basename( GK_GRAVITYCHARTS_PLUGIN_FILE );
		$this->_full_path    = GK_GRAVITYCHARTS_PLUGIN_FILE;
		$this->query_filters = new QueryFilters();

		/**
		 * Modifies the default capability required to access the plugin.
		 *
		 * @filter `gk/gravitycharts/capabilities/access`
		 *
		 * @since  1.0.2
		 *
		 * @param string $capability
		 */
		$this->_capabilities_form_settings = apply_filters( 'gk/gravitycharts/capabilities/access', $this->_capabilities_form_settings );

		add_filter( 'gk/foundation/integrations/helpscout/display', [ $this, 'maybe_display_helpscout_beacon' ] );

		parent::__construct();
	}

	/**
	 * Returns class instance.
	 *
	 * @since 1.0.4
	 *
	 * @return Chart_Feed
	 */
	public static function get_instance(): Chart_Feed {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new Chart_Feed();
		}

		return self::$_instance;
	}

	/**
	 * {@inheritdoc}
	 *
	 * @since 1.0
	 */
	public function init() {
		parent::init();

		add_filter( 'gform_noconflict_scripts', Closure::fromCallable( [ $this, 'allow_ui_assets' ] ) );
		add_filter( 'gform_noconflict_styles', Closure::fromCallable( [ $this, 'allow_ui_assets' ] ) );
		add_filter( 'gform_tooltips', Closure::fromCallable( [ $this, 'add_tooltips' ] ) );

		add_action( 'admin_enqueue_scripts', Closure::fromCallable( [ $this, 'configure_ui_translations' ] ) );
	}

	/**
	 * Sets minimum requirements to prevent bugs when using older versions, or missing dependencies.
	 *
	 * @since 1.0
	 *
	 * @return array
	 */
	public function minimum_requirements(): array {
		return [
			'php' => [
				'version' => GK_GRAVITYCHARTS_MIN_PHP_VERSION,
			],
		];
	}

	/**
	 * {@inheritdoc}
	 *
	 * @since 1.0
	 */
	public function get_menu_icon(): string {
		return '<svg style="height: 24px; width: 37px;" enable-background="new 0 0 630 630" height="630" viewBox="0 0 630 630" width="630" xmlns="http://www.w3.org/2000/svg"><path clip-rule="evenodd" d="m3.5 375.2v-264.2c0-6.6 4.9-12 12-12h23.9c6.2 0 11.9 5.4 11.9 12v264.2c0 6.4-5.7 12-11.9 12h-23.9c-7.1 0-12-5.6-12-12zm611.5 107.8h-530l188.6-188.1 87 87.5c4.7 4.7 12.3 4.7 17 0l246-246c4.7-4.7 4.7-12.3 0-17l-17-17c-4.7-4.7-12.3-4.7-17 0l-220.5 221.2-87.6-88.1c-4.7-4.7-12.3-4.7-17 0l-254.2 254.8c-4.8 4.2-6.8 10.3-6.8 16.7v12c0 6.6 4.9 12 12 12h599.5c6.6 0 12-5.4 12-12v-24c0-6.6-5.4-12-12-12z" fill-rule="evenodd"/></svg>';
	}

	/**
	 * {@inheritdoc}
	 *
	 * @since 1.0
	 *
	 * @return string
	 */
	public function feed_list_no_item_message(): string {
		return $this->empty_feeds_list_message();
	}

	/**
	 * Shows message when the feed list is empty.
	 *
	 * @since 1.0
	 *
	 * @param string|null $url URL to create the new feed.
	 *
	 * @return string The message.
	 */
	public function empty_feeds_list_message( $url = null ): string {
		$url = $url ? $url : add_query_arg( [ 'fid' => 0 ] );

		$output  = '<div class="gk-gravitycharts-no-feeds">';
		$output .= sprintf( '<img src="%s" width="150" height="205" alt="" style="padding: 0 20px 20px 20px; max-width: 40%%; display: block;" class="alignright" />', esc_url( plugins_url( 'images/floaty-sitting.svg', GK_GRAVITYCHARTS_PLUGIN_FILE ) ) );
		// translators: %1$s and %2$s are replaced by HTML.
		$output .= '<h5 style="font-size: 24px; font-weight: normal; max-width: 30rem; line-height: 1.4; padding: 10px; margin-top: 0; display: table-cell; vertical-align: middle; height: 185px; min-width: 60%">' . sprintf( esc_html__( "There are no charts configured for this form. Let's go %1\$screate one%2\$s!", 'gk-gravitycharts' ), "<a href='" . esc_url( $url ) . "'>", '</a>' ) . '</h5>';
		$output .= '</div>';

		return $output;
	}

	/**
	 * {@inheritdoc}
	 *
	 * @throws Exception Throws an exception if there's an invalid form object.
	 *
	 * @since 1.0
	 */
	public function scripts(): array {
		$script_file = 'build/form-settings.js';

		if ( ! file_exists( plugin_dir_path( GK_GRAVITYCHARTS_PLUGIN_FILE ) . $script_file ) ) {
			GravityKitFoundation::logger( 'gravitycharts', 'GravityCharts' )->error( "${script_file} not found - unable to load UI assets." );

			return parent::scripts();
		}

		// Sanity check for GF 2.5.
		if ( is_callable( [ $this, 'is_feed_edit_page' ] ) && $this->is_feed_edit_page() ) {
			$this->query_filters->set_form( $this->get_current_form() );
			$this->query_filters->enqueue_scripts(
				[
					'input_element_name' => '_gform_setting_conditional_logic',
					'conditions'         => rgar( $this->get_current_settings() ?? [], 'conditional_logic' ),
				]
			);
		}

		$scripts = [
			[
				'handle'    => 'gravitycharts_feed_settings',
				'src'       => plugin_dir_url( GK_GRAVITYCHARTS_PLUGIN_FILE ) . $script_file,
				'version'   => filemtime( plugin_dir_path( GK_GRAVITYCHARTS_PLUGIN_FILE ) . $script_file ),
				'in_footer' => false,
				'deps'      => [ 'wp-api-fetch', 'jquery-ui-draggable', 'jquery-ui-resizable' ],
				'strings'   => [
					'settingsMap'    => $this->settings_to_chartjs_options_map(),
					'defaultOptions' => $this->default_chartjs_options(),
					'colorPalettes'  => $this->get_color_palettes(),
				],
				'enqueue'   => [
					[
						'admin_page' => [ 'form_settings' ],
						'tab'        => $this->_slug,
					],
				],
			],
		];

		return array_merge( parent::scripts(), $scripts );
	}

	/**
	 * {@inheritdoc}
	 *
	 * @since 1.0
	 */
	public function styles(): array {
		$style_file = 'build/feed.css';

		if ( ! file_exists( plugin_dir_path( GK_GRAVITYCHARTS_PLUGIN_FILE ) . $style_file ) ) {
			GravityKitFoundation::logger( 'gravitycharts', 'GravityCharts' )->error( "${style_file} not found - unable to load UI assets." );

			return parent::scripts();
		}

		// Sanity check for GF 2.5.
		if ( is_callable( [ $this, 'is_feed_edit_page' ] ) && $this->is_feed_edit_page() ) {
			$this->query_filters->enqueue_styles();
		}

		$styles = [
			[
				'handle'  => 'gravitycharts_feed_settings',
				'src'     => plugin_dir_url( GK_GRAVITYCHARTS_PLUGIN_FILE ) . $style_file,
				'version' => filemtime( plugin_dir_path( GK_GRAVITYCHARTS_PLUGIN_FILE ) . $style_file ),
				'enqueue' => [
					[
						'admin_page' => [ 'form_settings' ],
						'tab'        => $this->_slug,
					],
				],
			],
		];

		return array_merge( parent::styles(), $styles );
	}

	/**
	 * Configures which columns should be displayed on the feed list page.
	 *
	 * @since 1.0
	 *
	 * @return array
	 * @uses  get_column_value_previewLink
	 */
	public function feed_list_columns(): array {
		return [
			'chartName'   => esc_html__( 'Name', 'gk-gravitycharts' ),
			'chartType'   => esc_html__( 'Chart Type', 'gk-gravitycharts' ),
			'shortcode'   => esc_html__( 'Shortcode', 'gk-gravitycharts' ),
			'previewLink' => esc_html__( 'Preview', 'gk-gravitycharts' ),
		];
	}

	/**
	 * Formats the value to be displayed in the preview column.
	 *
	 * @since   1.0
	 * @used-by GFFeedAddOn::get_column_value()
	 *
	 * @param array $feed The feed being included in the feed list.
	 *
	 * @return string
	 */
	public function get_column_value_chartType( array $feed ): string {

		$chart_type = Chart_Types::get( $feed['meta']['chartType'] );

		return $chart_type ? $chart_type['label'] : $feed['meta']['chartType'];
	}

	/**
	 * Formats the value to be displayed in the preview column.
	 *
	 * @since   1.0
	 * @used-by GFFeedAddOn::get_column_value()
	 *
	 * @param array $feed The feed being included in the feed list.
	 *
	 * @return string
	 */
	public function get_column_value_chartName( array $feed ): string {

		if ( '' !== rgars( $feed, 'meta/chartName' ) ) {
			return esc_html( $feed['meta']['chartName'] );
		}

		// translators: %d is replaced by the ID of the chart.
		return esc_html( sprintf( __( 'Chart #%d', 'gk-gravitycharts' ), $feed['id'] ) );
	}

	/**
	 * Formats the value to be displayed in the preview column.
	 *
	 * @since   1.0
	 * @used-by GFFeedAddOn::get_column_value()
	 *
	 * @param array $feed The feed being included in the feed list.
	 *
	 * @return string
	 */
	public function get_column_value_previewLink( array $feed ): string {
		$url = admin_url(
			sprintf(
				'admin.php?page=gf_edit_forms&view=settings&subview=%1$s&fid=%2$s&id=%3$s#%4$s',
				$this->_slug,
				$feed['id'],
				$feed['form_id'],
				'gform-settings-section-section-chart-preview'
			)
		);

		return sprintf(
			'<a href="%1$s">%2$s</a>',
			esc_url( $url ),
			esc_html__( 'Preview Chart', 'gk-gravitycharts' )
		);
	}

	/**
	 * Displays the chart's shortcode in the preview column.
	 *
	 * @since   1.1
	 *
	 * @param array $feed The feed being included in the feed list.
	 *
	 * @return string
	 */
	public function get_column_value_shortcode( array $feed ): string {
		return sprintf(
			'<input class="code" readonly value="[%s id=&quot;%s&quot;]" />',
			Shortcode::SHORTCODE,
			$feed['id']
		);
	}

	/**
	 * Gets color palettes used to customize Chart.js.
	 *
	 * "Qualitative colors are used to render categorical data, such as gender or race, that has no inherent sequential order."
	 *
	 * This product includes color specifications and designs developed by Cynthia Brewer (http://colorbrewer.org/).
	 *
	 * @since     1.0
	 *
	 * @return array
	 * @copyright Copyright (c) 2002 Cynthia Brewer, Mark Harrower, and The Pennsylvania State University.
	 */
	public static function get_color_palettes(): array {
		$color_palettes = [
			'default'             => [
				'label'                => esc_html__( 'Default', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(0,160,210)',
				'borderColor'          => [
					'rgb(0,160,210)',
					'rgb(130,110,180)',
					'rgb(70,180,80)',
					'rgb(255,185,0)',
					'rgb(245,110,40)',
					'rgb(220,50,50)',
				],
				'backgroundColor'      => [
					'rgb(0,160,210,0.85)',
					'rgb(130,110,180,0.85)',
					'rgb(70,180,80,0.85)',
					'rgb(255,185,0,0.85)',
					'rgb(245,110,40,0.85)',
					'rgb(220,50,50,0.85)',
				],
			],
			'google_standard'     => [
				'label'                => esc_html__( 'Standard', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(66, 133, 244)',
				'borderColor'          => [
					'rgb(66, 133, 244)',
					'rgb(234, 67, 53)',
					'rgb(251, 188, 4)',
					'rgb(52, 168, 83)',
					'rgb(255, 109, 1)',
					'rgb(70, 189, 198)',
				],
				'backgroundColor'      => [
					'rgba(66, 133, 244,0.85)',
					'rgba(234, 67, 53,0.85)',
					'rgba(251, 188, 4,0.85)',
					'rgba(52, 168, 83,0.85)',
					'rgba(255, 109, 1,0.85)',
					'rgba(70, 189, 198,0.85)',
				],
			],
			'google_simple_light' => [
				'label'                => esc_html__( 'Simple Light', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(88, 145, 173)',
				'borderColor'          => [
					'rgb(88, 145, 173)',
					'rgb(0, 69, 97)',
					'rgb(255, 111, 49)',
					'rgb(28, 118, 133)',
					'rgb(15, 69, 168)',
					'rgb(76, 220, 139)',
				],
				'backgroundColor'      => [
					'rgba(88, 145, 173,0.85)',
					'rgba(0, 69, 97,0.85)',
					'rgba(255, 111, 49,0.85)',
					'rgba(28, 118, 133,0.85)',
					'rgba(15, 69, 168,0.85)',
					'rgba(76, 220, 139,0.85)',
				],
			],
			'google_streamline'   => [
				'label'                => esc_html__( 'Streamline', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(26, 153, 136)',
				'borderColor'          => [
					'rgb(26, 153, 136)',
					'rgb(45, 114, 157)',
					'rgb(31, 62, 120)',
					'rgb(235, 86, 0)',
					'rgb(255, 153, 172)',
					'rgb(255, 212, 184)',
				],
				'backgroundColor'      => [
					'rgba(26, 153, 136,0.85)',
					'rgba(45, 114, 157,0.85)',
					'rgba(31, 62, 120,0.85)',
					'rgba(235, 86, 0,0.85)',
					'rgba(255, 153, 172,0.85)',
					'rgba(255, 212, 184,0.85)',
				],
			],
			'google_paradigm'     => [
				'label'                => esc_html__( 'Paradigm', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(111, 200, 214)',
				'borderColor'          => [
					'rgb(111, 200, 214)',
					'rgb(0, 47, 74)',
					'rgb(173, 132, 99)',
					'rgb(184, 87, 65)',
					'rgb(0, 147, 132)',
					'rgb(237, 218, 201)',
				],
				'backgroundColor'      => [
					'rgba(111, 200, 214,0.85)',
					'rgba(0, 47, 74,0.85)',
					'rgba(173, 132, 99,0.85)',
					'rgba(184, 87, 65,0.85)',
					'rgba(0, 147, 132,0.85)',
					'rgba(237, 218, 201,0.85)',
				],
			],
			'google_shift'        => [
				'label'                => esc_html__( 'Shift', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(20, 76, 245)',
				'borderColor'          => [
					'rgb(20, 76, 245)',
					'rgb(0, 121, 107)',
					'rgb(0, 67, 94)',
					'rgb(191, 134, 89)',
					'rgb(217, 86, 63)',
					'rgb(231, 187, 99)',
				],
				'backgroundColor'      => [
					'rgba(20, 76, 245,0.85)',
					'rgba(0, 121, 107,0.85)',
					'rgba(0, 67, 94,0.85)',
					'rgba(191, 134, 89,0.85)',
					'rgba(217, 86, 63,0.85)',
					'rgba(231, 187, 99,0.85)',
				],
			],
			'google_momentum'     => [
				'label'                => esc_html__( 'Momentum', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(215, 230, 163)',
				'borderColor'          => [
					'rgb(215, 230, 163)',
					'rgb(192, 121, 27)',
					'rgb(253, 91, 88)',
					'rgb(11, 99, 116)',
					'rgb(39, 39, 139)',
					'rgb(141, 216, 211)',
				],
				'backgroundColor'      => [
					'rgba(215, 230, 163,0.85)',
					'rgba(192, 121, 27,0.85)',
					'rgba(253, 91, 88,0.85)',
					'rgba(11, 99, 116,0.85)',
					'rgba(39, 39, 139,0.85)',
					'rgba(141, 216, 211,0.85)',
				],
			],
			'google_earthy'       => [
				'label'                => esc_html__( 'Earthy', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(139, 137, 72)',
				'borderColor'          => [
					'rgb(139, 137, 72)',
					'rgb(176, 114, 93)',
					'rgb(231, 154, 60)',
					'rgb(143, 55, 56)',
					'rgb(68, 120, 116)',
					'rgb(210, 212, 121)',
				],
				'backgroundColor'      => [
					'rgba(139, 137, 72,0.85)',
					'rgba(176, 114, 93,0.85)',
					'rgba(231, 154, 60,0.85)',
					'rgba(143, 55, 56,0.85)',
					'rgba(68, 120, 116,0.85)',
					'rgba(210, 212, 121,0.85)',
				],
			],
			'google_energetic'    => [
				'label'                => esc_html__( 'Energetic', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(209, 235, 96)',
				'borderColor'          => [
					'rgb(209, 235, 96)',
					'rgb(29, 35, 132)',
					'rgb(245, 166, 155)',
					'rgb(228, 72, 25)',
					'rgb(166, 128, 87)',
					'rgb(244, 170, 17)',
				],
				'backgroundColor'      => [
					'rgba(209, 235, 96,0.85)',
					'rgba(29, 35, 132,0.85)',
					'rgba(245, 166, 155,0.85)',
					'rgba(228, 72, 25,0.85)',
					'rgba(166, 128, 87,0.85)',
					'rgba(244, 170, 17,0.85)',
				],
			],
			'google_ocean'        => [
				'label'                => esc_html__( 'Ocean', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(143, 225, 255)',
				'borderColor'          => [
					'rgb(143, 225, 255)',
					'rgb(0, 120, 184)',
					'rgb(0, 163, 250)',
					'rgb(0, 145, 222)',
					'rgb(66, 189, 255)',
					'rgb(110, 204, 255)',
				],
				'backgroundColor'      => [
					'rgba(143, 225, 255,0.85)',
					'rgba(0, 120, 184,0.85)',
					'rgba(0, 163, 250,0.85)',
					'rgba(0, 145, 222,0.85)',
					'rgba(66, 189, 255,0.85)',
					'rgba(110, 204, 255,0.85)',
				],
			],
			'google_cozy'         => [
				'label'                => esc_html__( 'Cozy', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(56, 98, 181)',
				'borderColor'          => [
					'rgb(56, 98, 181)',
					'rgb(0, 83, 92)',
					'rgb(159, 84, 92)',
					'rgb(205, 132, 14)',
					'rgb(217, 193, 173)',
					'rgb(213, 232, 172)',
				],
				'backgroundColor'      => [
					'rgba(56, 98, 181,0.85)',
					'rgba(0, 83, 92,0.85)',
					'rgba(159, 84, 92,0.85)',
					'rgba(205, 132, 14,0.85)',
					'rgba(217, 193, 173,0.85)',
					'rgba(213, 232, 172,0.85)',
				],
			],
			'google_groovy'       => [
				'label'                => esc_html__( 'Groovy', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(31, 149, 209)',
				'borderColor'          => [
					'rgb(31, 149, 209)',
					'rgb(201, 50, 50)',
					'rgb(13, 57, 140)',
					'rgb(226, 114, 40)',
					'rgb(252, 68, 121)',
					'rgb(220, 220, 18)',
				],
				'backgroundColor'      => [
					'rgba(31, 149, 209,0.85)',
					'rgba(201, 50, 50,0.85)',
					'rgba(13, 57, 140,0.85)',
					'rgba(226, 114, 40,0.85)',
					'rgba(252, 68, 121,0.85)',
					'rgba(220, 220, 18,0.85)',
				],
			],
			'google_calm'         => [
				'label'                => esc_html__( 'Calm', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(230, 140, 165)',
				'borderColor'          => [
					'rgb(230, 140, 165)',
					'rgb(96, 143, 102)',
					'rgb(223, 163, 152)',
					'rgb(39, 71, 102)',
					'rgb(171, 144, 132)',
					'rgb(97, 57, 66)',
				],
				'backgroundColor'      => [
					'rgba(230, 140, 165,0.85)',
					'rgba(96, 143, 102,0.85)',
					'rgba(223, 163, 152,0.85)',
					'rgba(39, 71, 102,0.85)',
					'rgba(171, 144, 132,0.85)',
					'rgba(97, 57, 66,0.85)',
				],
			],
			'google_forest'       => [
				'label'                => esc_html__( 'Forest', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(160, 219, 193)',
				'borderColor'          => [
					'rgb(160, 219, 193)',
					'rgb(15, 107, 79)',
					'rgb(93, 166, 138)',
					'rgb(56, 138, 102)',
					'rgb(118, 181, 154)',
					'rgb(140, 207, 172)',
				],
				'backgroundColor'      => [
					'rgba(160, 219, 193,0.85)',
					'rgba(15, 107, 79,0.85)',
					'rgba(93, 166, 138,0.85)',
					'rgba(56, 138, 102,0.85)',
					'rgba(118, 181, 154,0.85)',
					'rgba(140, 207, 172,0.85)',
				],
			],
			'google_vintage'      => [
				'label'                => esc_html__( 'Vintage', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(155, 187, 170)',
				'borderColor'          => [
					'rgb(155, 187, 170)',
					'rgb(212, 84, 27)',
					'rgb(222, 151, 121)',
					'rgb(0, 84, 84)',
					'rgb(137, 94, 33)',
					'rgb(129, 148, 98)',
				],
				'backgroundColor'      => [
					'rgba(155, 187, 170,0.85)',
					'rgba(212, 84, 27,0.85)',
					'rgba(222, 151, 121,0.85)',
					'rgba(0, 84, 84,0.85)',
					'rgba(137, 94, 33,0.85)',
					'rgba(129, 148, 98,0.85)',
				],
			],
			'google_retro'        => [
				'label'                => esc_html__( 'Retro', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(245, 149, 156)',
				'borderColor'          => [
					'rgb(245, 149, 156)',
					'rgb(0, 99, 145)',
					'rgb(254, 185, 41)',
					'rgb(226, 74, 56)',
					'rgb(40, 153, 139)',
					'rgb(192, 222, 0)',
				],
				'backgroundColor'      => [
					'rgba(245, 149, 156,0.85)',
					'rgba(0, 99, 145,0.85)',
					'rgba(254, 185, 41,0.85)',
					'rgba(226, 74, 56,0.85)',
					'rgba(40, 153, 139,0.85)',
					'rgba(192, 222, 0,0.85)',
				],
			],
			'google_coral'        => [
				'label'                => esc_html__( 'Coral', 'gk-gravitycharts' ),
				'nature'               => 'qualitative',
				'pointBackgroundColor' => 'rgb(155, 204, 198)',
				'borderColor'          => [
					'rgb(155, 204, 198)',
					'rgb(215, 118, 89)',
					'rgb(0, 69, 82)',
					'rgb(171, 64, 40)',
					'rgb(69, 125, 124)',
					'rgb(80, 166, 166)',
				],
				'backgroundColor'      => [
					'rgba(155, 204, 198,0.85)',
					'rgba(215, 118, 89,0.85)',
					'rgba(0, 69, 82,0.85)',
					'rgba(171, 64, 40,0.85)',
					'rgba(69, 125, 124,0.85)',
					'rgba(80, 166, 166,0.85)',
				],
			],
			'chartJS'             => [
				'label'           => 'Chart.js',
				'nature'          => 'qualitative',
				'backgroundColor' => [
					'rgb(255, 99, 132, 0.85)',
					'rgb(255, 159, 64, 0.85)',
					'rgb(255, 205, 86, 0.85)',
					'rgb(75, 192, 192, 0.85)',
					'rgb(54, 162, 235, 0.85)',
					'rgb(153, 102, 255, 0.85)',
					'rgb(201, 203, 207, 0.85)',
				],
				'borderColor'     => [
					'rgb(255, 99, 132)',
					'rgb(255, 159, 64)',
					'rgb(255, 205, 86)',
					'rgb(75, 192, 192)',
					'rgb(54, 162, 235)',
					'rgb(153, 102, 255)',
					'rgb(201, 203, 207)',
				],
			],
			'paired'              => [
				'nature'               => 'qualitative',
				'label'                => esc_html__( 'Paired', 'gk-gravitycharts' ),
				'pointBackgroundColor' => 'rgb(166,206,227)',
				'borderColor'          => [
					'rgb(166,206,227)',
					'rgb(31,120,180)',
					'rgb(178,223,138)',
					'rgb(51,160,44)',
					'rgb(251,154,153)',
					'rgb(227,26,28)',
					'rgb(253,191,111)',
					'rgb(255,127,0)',
					'rgb(202,178,214)',
				],
				'backgroundColor'      => [
					'rgb(166,206,227,0.85)',
					'rgb(31,120,180,0.85)',
					'rgb(178,223,138,0.85)',
					'rgb(51,160,44,0.85)',
					'rgb(251,154,153,0.85)',
					'rgb(227,26,28,0.85)',
					'rgb(253,191,111,0.85)',
					'rgb(255,127,0,0.85)',
					'rgb(202,178,214,0.85)',
				],
			],
			'set1'                => [
				'nature'               => 'qualitative',
				'label'                => esc_html__( 'Bright', 'gk-gravitycharts' ),
				'pointBackgroundColor' => 'rgb(228,26,28)',
				'borderColor'          => [
					'rgb(228,26,28)',
					'rgb(55,126,184)',
					'rgb(77,175,74)',
					'rgb(152,78,163)',
					'rgb(255,127,0)',
					'rgb(255,255,51)',
					'rgb(166,86,40)',
					'rgb(247,129,191)',
					'rgb(153,153,153)',
				],
				'backgroundColor'      => [
					'rgb(228,26,28,0.85)',
					'rgb(55,126,184,0.85)',
					'rgb(77,175,74,0.85)',
					'rgb(152,78,163,0.85)',
					'rgb(255,127,0,0.85)',
					'rgb(255,255,51,0.85)',
					'rgb(166,86,40,0.85)',
					'rgb(247,129,191,0.85)',
					'rgb(153,153,153,0.85)',
				],
			],
			'learnui_qualitative' => [
				'nature'               => 'qualitative',
				'label'                => esc_html__( 'LearnUI', 'gk-gravitycharts' ),
				'pointBackgroundColor' => 'rgb(0, 63, 92)',
				'borderColor'          => [
					'rgb(0, 63, 92)',
					'rgb(47, 75, 124)',
					'rgb(102, 81, 145)',
					'rgb(160, 81, 149)',
					'rgb(212, 80, 135)',
					'rgb(249, 93, 106)',
					'rgb(255, 124, 67)',
					'rgb(255, 166, 0)',
				],
				'backgroundColor'      => [
					'rgb(0, 63, 92, 0.85)',
					'rgb(47, 75, 124, 0.85)',
					'rgb(102, 81, 145, 0.85)',
					'rgb(160, 81, 149, 0.85)',
					'rgb(212, 80, 135, 0.85)',
					'rgb(249, 93, 106, 0.85)',
					'rgb(255, 124, 67, 0.85)',
					'rgb(255, 166, 0, 0.85)',
				],
			],
			'census_qualitative'  => [
				'nature'               => 'qualitative',
				'label'                => esc_html__( 'Census', 'gk-gravitycharts' ),
				'pointBackgroundColor' => 'rgb(0, 149, 168)',
				'borderColor'          => [
					'rgb(0, 149, 168)',
					'rgb(17, 46, 81)',
					'rgb(255, 112, 67)',
					'rgb(120, 144, 156)',
					'rgb(46, 120, 210)',
					'rgb(0, 108, 122)',
					'rgb(255, 151, 118)',
				],
				'backgroundColor'      => [
					'rgb(0, 149, 168, 0.85)',
					'rgb(17, 46, 81, 0.85)',
					'rgb(255, 112, 67, 0.85)',
					'rgb(120, 144, 156, 0.85)',
					'rgb(46, 120, 210, 0.85)',
					'rgb(0, 108, 122, 0.85)',
					'rgb(255, 151, 118, 0.85)',
				],
			],
			'census_teal'         => [
				'nature'               => 'sequential',
				// translators: %s is replaced with color name (e.g., Blue).
				'label'                => sprintf( esc_html__( 'Census %s', 'gk-gravitycharts' ), esc_html__( 'Teal', 'gk-gravitycharts' ) ),
				'pointBackgroundColor' => 'rgb(0, 40, 46)',
				'borderColor'          => [
					'rgb(0, 40, 46)',
					'rgb(0, 72, 81)',
					'rgb(0, 108, 122)',
					'rgb(0, 149, 168)',
					'rgb(0, 190, 214)',
					'rgb(99, 225, 234)',
					'rgb(168, 245, 255)',
				],
				'backgroundColor'      => [
					'rgb(0, 40, 46, 0.85 )',
					'rgb(0, 72, 81, 0.85 )',
					'rgb(0, 108, 122, 0.85 )',
					'rgb(0, 149, 168, 0.85 )',
					'rgb(0, 190, 214, 0.85 )',
					'rgb(99, 225, 234, 0.85 )',
					'rgb(168, 245, 255, 0.85 )',
				],
			],
			'census_blue'         => [
				'nature'               => 'sequential',
				// translators: %s is replaced with color name (e.g., Blue).
				'label'                => sprintf( esc_html__( 'Census %s', 'gk-gravitycharts' ), esc_html__( 'Blue', 'gk-gravitycharts' ) ),
				'pointBackgroundColor' => 'rgb(8, 22, 39)',
				'borderColor'          => [
					'rgb(8, 22, 39)',
					'rgb(17, 46, 81)',
					'rgb(32, 84, 147)',
					'rgb(46, 120, 210)',
					'rgb(109, 161, 224)',
					'rgb(151, 188, 233)',
					'rgb(193, 215, 242)',
				],
				'backgroundColor'      => [
					'rgb(8, 22, 39, 0.85)',
					'rgb(17, 46, 81, 0.85)',
					'rgb(32, 84, 147, 0.85)',
					'rgb(46, 120, 210, 0.85)',
					'rgb(109, 161, 224, 0.85)',
					'rgb(151, 188, 233, 0.85)',
					'rgb(193, 215, 242, 0.85)',
				],
			],
			'census_orange'       => [
				'nature'               => 'sequential',
				// translators: %s is replaced with color name (e.g., Blue).
				'label'                => sprintf( esc_html__( 'Census %s', 'gk-gravitycharts' ), esc_html__( 'Orange', 'gk-gravitycharts' ) ),
				'pointBackgroundColor' => 'rgb(93, 40, 24)',
				'borderColor'          => [
					'rgb(93, 40, 24)',
					'rgb(133, 58, 34)',
					'rgb(194, 84, 50)',
					'rgb(255, 112, 67)',
					'rgb(255, 151, 118)',
					'rgb(255, 190, 169)',
					'rgb(255, 228, 220)',
				],
				'backgroundColor'      => [
					'rgb(93, 40, 24, 0.85)',
					'rgb(133, 58, 34, 0.85)',
					'rgb(194, 84, 50, 0.85)',
					'rgb(255, 112, 67, 0.85)',
					'rgb(255, 151, 118, 0.85)',
					'rgb(255, 190, 169, 0.85)',
					'rgb(255, 228, 220, 0.85)',
				],
			],
			'census_grey'         => [
				'nature'               => 'sequential',
				// translators: %s is replaced with color name (e.g., Blue).
				'label'                => sprintf( esc_html__( 'Census %s', 'gk-gravitycharts' ), esc_html__( 'Grey', 'gk-gravitycharts' ) ),
				'pointBackgroundColor' => 'rgb(34, 44, 49)',
				'borderColor'          => [
					'rgb(34, 44, 49)',
					'rgb(54, 72, 80)',
					'rgb(75, 99, 110)',
					'rgb(120, 144, 156)',
					'rgb(167, 192, 205)',
					'rgb(200, 215, 223)',
					'rgb(232, 239, 242)',
				],
				'backgroundColor'      => [
					'rgb(34, 44, 49, 0.85)',
					'rgb(54, 72, 80, 0.85)',
					'rgb(75, 99, 110, 0.85)',
					'rgb(120, 144, 156, 0.85)',
					'rgb(167, 192, 205, 0.85)',
					'rgb(200, 215, 223, 0.85)',
					'rgb(232, 239, 242, 0.85)',
				],
			],
		];

		/**
		 * Modifies the default colors used when creating a chart.
		 *
		 * @filter `gk/gravitycharts/color-palettes`
		 *
		 * @since  1.0
		 *
		 * @see    https://www.chartjs.org/docs/latest/charts/line.html for example settings.
		 * @see    https://xdgov.github.io/data-design-standards/components/colors for accessible color guidance.
		 *
		 * @param array $color_palettes Chart.js settings presets.
		 */
		return apply_filters( 'gk/gravitycharts/color-palettes', $color_palettes );
	}

	/**
	 * Configures the settings which should be rendered on the feed edit page under Form Settings > GravityCharts.
	 *
	 * @since 1.0
	 *
	 * @return array
	 */
	public function feed_settings_fields(): array {
		$fields = [
			[
				'fields' => [
					[
						'name'     => 'chartName',
						'label'    => esc_html__( 'Chart Name', 'gk-gravitycharts' ),
						'required' => false,
						'type'     => 'text',
						'class'    => 'small',
						'tooltip'  => sprintf(
							'<h6>%s</h6> %s',
							esc_html__( 'Feed Name', 'gk-gravitycharts' ),
							esc_html__( 'Enter a feed name to uniquely identify this configuration.', 'gk-gravitycharts' )
						),
					],
					[
						'name'          => 'chartType',
						'label'         => esc_html__( 'Chart Type', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'required'      => true,
						'class'         => 'gravitycharts-option',
						'default_value' => Chart_Types::DEFAULT_TYPE,
						'choices'       => Chart_Types::get_all(),
					],
				],
			],
			[
				'title'  => esc_html__( 'Chart Data', 'gk-gravitycharts' ),
				'fields' => [
					[
						'name'        => 'dataField',
						'label'       => esc_html__( 'Data Source', 'gk-gravitycharts' ),
						'description' => esc_html__( 'Choose a field to be charted. Limited field types are currently supported.', 'gk-gravitycharts' ),
						'tooltip'     => 'gravitycharts_supported_fields',
						'type'        => 'field_select',
						'required'    => true,
						'args'        => [
							'input_types' => [ 'checkbox', 'select', 'radio', 'multiselect', 'likert', 'rank', 'rating', 'survey', 'quiz' ],
						],
					],
					[
						'name'        => 'dataCondition',
						'full_screen' => false,
						'label'       => esc_html__( 'Conditional Logic', 'gk-gravitycharts' ),
						'tooltip'     => 'export_conditional_logic',
						'type'        => 'html',
						'html'        => '<div id="gk-query-filters"></div>',
					],
				],
			],
			[
				'title'  => esc_html__( 'Chart Colors', 'gk-gravitycharts' ),
				'class'  => 'gravitycharts-options-section',
				'fields' => [
					[
						'name'          => 'palette',
						'label'         => esc_html__( 'Color Palette', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'class'         => 'gravitycharts-option small color-grid',
						'default_value' => 'default',
						'choices'       => $this->get_palette_choices(),
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'palette' ),
								],
							],
						],
					],
				],
			],
			[
				'title'  => esc_html__( 'Chart Styles', 'gk-gravitycharts' ),
				'class'  => 'gravitycharts-options-section',
				'fields' => [
					[
						'name'          => 'indexAxis',
						'label'         => esc_html__( 'Bar Direction', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'choices'       => [
							[
								'label' => esc_html__( 'Vertical', 'gk-gravitycharts' ),
								'value' => 'x',
								'icon'  => Chart_Types::get( 'bar' )['icon'],
							],
							[
								'label' => esc_html__( 'Horizontal', 'gk-gravitycharts' ),
								'value' => 'y',
								'icon'  => Chart_Types::get( 'bar' )['icon'],
							],
						],
						'class'         => 'gravitycharts-option small',
						'default_value' => 'x',
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'indexAxis' ),
								],
							],
						],
					],
					[
						'name'          => 'tension',
						'label'         => esc_html__( 'Line Tension', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'horizontal'    => true,
						'class'         => 'gravitycharts-option',
						'choices'       => [
							[
								'label' => esc_html__( 'No Curve', 'gk-gravitycharts' ),
								'value' => 0,
							],
							[
								'label' => esc_html__( 'Light Curve', 'gk-gravitycharts' ),
								'value' => 0.1,
							],
							[
								'label' => esc_html__( 'Medium Curve', 'gk-gravitycharts' ),
								'value' => 0.25,
							],
							[
								'label' => esc_html__( 'Strong Curve', 'gk-gravitycharts' ),
								'value' => 0.5,
							],
						],
						'default_value' => 0,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'tension' ),
								],
							],
						],
					],
					[
						'name'          => 'cutout',
						'label'         => esc_html__( 'Cutout', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'horizontal'    => true,
						'class'         => 'gravitycharts-option',
						'default_value' => '0',
						'choices'       => [
							[
								'label' => esc_html__( 'None (Pie&nbsp;Chart)', 'gk-gravitycharts' ),
								'value' => '0',
							],
							[
								'label' => esc_html__( 'Small', 'gk-gravitycharts' ),
								'value' => '20%',
							],
							[
								'label' => esc_html__( 'Medium', 'gk-gravitycharts' ),
								'value' => '40%',
							],
							[
								'label' => esc_html__( 'Large', 'gk-gravitycharts' ),
								'value' => '60%',
							],
							[
								'label' => esc_html__( 'Extra-Large', 'gk-gravitycharts' ),
								'value' => '80%',
							],
						],
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'cutout' ),
								],
							],
						],
					],
					[
						'name'          => 'offset',
						'label'         => esc_html__( 'Gap', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'horizontal'    => true,
						'class'         => 'gravitycharts-option',
						'default_value' => '24',
						'choices'       => [
							[
								'label' => esc_html__( 'No Gap', 'gk-gravitycharts' ),
								'value' => '0',
							],
							[
								'label' => esc_html__( 'Small', 'gk-gravitycharts' ),
								'value' => '24',
							],
							[
								'label' => esc_html__( 'Medium', 'gk-gravitycharts' ),
								'value' => '48',
							],
							[
								'label' => esc_html__( 'Large', 'gk-gravitycharts' ),
								'value' => '96',
							],
							[
								'label' => esc_html__( 'Extra-Large', 'gk-gravitycharts' ),
								'value' => '192',
							],
						],
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'offset' ),
								],
							],
						],
					],
					[
						'name'          => 'borderWidth',
						'label'         => esc_html__( 'Border Width', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'horizontal'    => true,
						'class'         => 'gravitycharts-option',
						'default_value' => '6',
						'choices'       => [
							[
								'label' => esc_html__( 'No Border', 'gk-gravitycharts' ),
								'value' => '0',
							],
							[
								'label' => esc_html__( 'Thin', 'gk-gravitycharts' ),
								'value' => '3',
							],
							[
								'label' => esc_html__( 'Medium', 'gk-gravitycharts' ),
								'value' => '6',
							],
							[
								'label' => esc_html__( 'Thick', 'gk-gravitycharts' ),
								'value' => '12',
							],
						],
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'borderWidth' ),
								],
							],
						],
					],
					[
						'name'          => 'pointRadius',
						'label'         => esc_html__( 'Point Size', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'horizontal'    => true,
						'class'         => 'gravitycharts-option',
						'default_value' => '6',
						'choices'       => [
							[
								'label' => esc_html__( 'No Points', 'gk-gravitycharts' ),
								'value' => '0',
							],
							[
								'label' => esc_html__( 'Small', 'gk-gravitycharts' ),
								'value' => '6',
							],
							[
								'label' => esc_html__( 'Medium', 'gk-gravitycharts' ),
								'value' => '12',
							],
							[
								'label' => esc_html__( 'Large', 'gk-gravitycharts' ),
								'value' => '18',
							],
						],
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'pointRadius' ),
								],
							],
						],
					],
					[
						'name'          => 'pointStyle',
						'label'         => esc_html__( 'Point Style', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'class'         => 'gravitycharts-option small',
						'default_value' => 'circle',
						'choices'       => [
							[
								'label' => esc_html__( 'Circle', 'gk-gravitycharts' ),
								'value' => 'circle',
								'icon'  => '<svg role="img" viewBox="0 0 12 12"><circle cx="6" cy="6" r="6"></circle></svg>',
							],
							[
								'label' => esc_html__( 'Square', 'gk-gravitycharts' ),
								'value' => 'rect',
								'icon'  => '<svg role="img" viewBox="0 0 12 12"><rect x="0" y="0" width="12" height="12"></rect></svg>',
							],
							[
								'label' => esc_html__( 'Round Rect', 'gk-gravitycharts' ),
								'value' => 'rectRounded',
								'icon'  => '<svg role="img" viewBox="0 0 12 12" xmlns="http://www.w3.org/2000/svg" width="12" height="12"><path d="M9 12H3c-1.7 0-3-1.3-3-3V3c0-1.7 1.3-3 3-3h6c1.7 0 3 1.3 3 3v6c0 1.7-1.3 3-3 3z"/></svg>',
							],
							[
								'label' => esc_html__( 'Diamond', 'gk-gravitycharts' ),
								'value' => 'rectRot',
								'icon'  => '<svg role="img" viewBox="0 0 12 12"><path d="M 6,0 L 12,6 L 6,12 L 0,6 z"></path></svg>',
							],
							[
								'label' => esc_html__( 'Triangle', 'gk-gravitycharts' ),
								'value' => 'triangle',
								'icon'  => '<svg role="img" viewBox="0 0 12 12"><path d="M 6,0 L 12,12 L 0,12"></path></svg>',
							],
						],
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'pointStyle' ),
								],
								[
									'field'  => 'pointRadius',
									'values' => [ '6', '12', '18' ],
								],
							],
						],
					],
				],
			],
			[
				'title'  => esc_html__( 'Axis', 'gk-gravitycharts' ),
				'class'  => 'gravitycharts-options-section',
				'fields' => [
					[
						'name'          => 'angleLinesDisplay',
						'label'         => esc_html__( 'Show Angle Lines', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => true,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'angleLinesDisplay' ),
								],
							],
						],
					],
					[
						'name'          => 'xGridDisplay',
						'label'         => esc_html__( 'Show X Axis Grid Lines', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => true,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'xGridDisplay' ),
								],
							],
						],
					],
					[
						'name'          => 'yGridDisplay',
						'label'         => esc_html__( 'Show Y Axis Grid Lines', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => true,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'yGridDisplay' ),
								],
							],
						],
					],
					[
						'name'          => 'rGridDisplay',
						'label'         => esc_html__( 'Show Grid Lines', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => true,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'rGridDisplay' ),
								],
							],
						],
					],
					[
						'name'          => 'ticksDisplay',
						'label'         => esc_html__( 'Show Scale Labels', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => true,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'ticksDisplay' ),
								],
							],
						],
					],
					[
						'name'          => 'pointLabelsDisplay',
						'label'         => esc_html__( 'Show Point Labels', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => true,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'pointLabelsDisplay' ),
								],
							],
						],
					],
					[
						'name'          => 'autoScale',
						'label'         => esc_html__( 'Auto Scale', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => true,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'autoScale' ),
								],
							],
						],
					],
					[
						'name'          => 'min',
						'label'         => esc_html__( 'Scale Min', 'gk-gravitycharts' ),
						'type'          => 'text',
						'input_type'    => 'number',
						'autocomplete'  => 'off',
						'min'           => 0,
						'class'         => 'gravitycharts-option scale',
						'default_value' => 0,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'min' ),
								],
								[
									'field'  => 'autoScale',
									'values' => [ false ],
								],
							],
						],
					],
					[
						'name'          => 'max',
						'label'         => esc_html__( 'Scale Max', 'gk-gravitycharts' ),
						'type'          => 'text',
						'input_type'    => 'number',
						'autocomplete'  => 'off',
						'min'           => 1,
						'class'         => 'gravitycharts-option scale',
						'default_value' => 1,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field'  => 'chartType',
									'values' => Chart_Types::supports_setting( 'max' ),
								],
								[
									'field'  => 'autoScale',
									'values' => [ false ],
								],
							],
						],
					],
				],
			],
			[
				'title'  => esc_html__( 'Legend', 'gk-gravitycharts' ),
				'class'  => 'gravitycharts-options-section',
				'fields' => [
					[
						'name'          => 'titleDisplay',
						'label'         => esc_html__( 'Show Chart Title', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => false,
					],
					[
						'name'          => 'titleText',
						'label'         => esc_html__( 'Chart Title Text', 'gk-gravitycharts' ),
						'type'          => 'text',
						'class'         => 'gravitycharts-option',
						'default_value' => '',
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field' => 'titleDisplay',
								],
							],
						],
					],
					[
						'name'          => 'titleSize',
						'label'         => esc_html__( 'Title Font Size', 'gk-gravitycharts' ),
						'type'          => 'text',
						'input_type'    => 'number',
						'min'           => 10,
						'max'           => 96,
						'class'         => 'gravitycharts-option',
						'default_value' => 22,
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field' => 'titleDisplay',
								],
							],
						],
					],
					[
						'name'          => 'legendDisplay',
						'label'         => esc_html__( 'Enable Legend', 'gk-gravitycharts' ),
						'type'          => 'toggle',
						'class'         => 'gravitycharts-option',
						'default_value' => true,
					],
					[
						'name'          => 'legendPosition',
						'label'         => esc_html__( 'Legend Position', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'horizontal'    => true,
						'class'         => 'gravitycharts-option legend',
						'default_value' => 'top',
						'choices'       => [
							[
								'label' => esc_html__( 'Top', 'gk-gravitycharts' ),
								'value' => 'top',
							],
							[
								'label' => esc_html__( 'Bottom', 'gk-gravitycharts' ),
								'value' => 'bottom',
							],
							[
								'label' => esc_html__( 'Left', 'gk-gravitycharts' ),
								'value' => 'left',
							],
							[
								'label' => esc_html__( 'Right', 'gk-gravitycharts' ),
								'value' => 'right',
							],
						],
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field' => 'legendDisplay',
								],
							],
						],
					],
					[
						'name'          => 'legendAlign',
						'label'         => esc_html__( 'Legend Alignment', 'gk-gravitycharts' ),
						'type'          => 'radio',
						'horizontal'    => true,
						'class'         => 'gravitycharts-option legend',
						'default_value' => 'center',
						'choices'       => [
							[
								'label' => esc_html__( 'Start', 'gk-gravitycharts' ),
								'value' => 'start',
							],
							[
								'label' => esc_html__( 'Center', 'gk-gravitycharts' ),
								'value' => 'center',
							],
							[
								'label' => esc_html__( 'End', 'gk-gravitycharts' ),
								'value' => 'end',
							],
						],
						'dependency'    => [
							'live'   => true,
							'fields' => [
								[
									'field' => 'legendDisplay',
								],
							],
						],
					],
				],
			],
			[
				'title'  => esc_html__( 'Chart Configuration', 'gk-gravitycharts' ),
				'id'     => 'section-chart-preview',
				'fields' => [
					[
						'name' => 'chartPreview',
						'type' => 'chart_preview',
					],
				],
			],
		];

		/**
		 * Modifies feed field configuration.
		 *
		 * @filter `gk/gravitycharts/feed/settings-fields`
		 *
		 * @since  1.0
		 *
		 * @param array $fields
		 */
		return apply_filters( 'gk/gravitycharts/feed/settings-fields', $fields );
	}

	/**
	 * Returns an array mapping the GravityCharts feed settings fields to Chart.js properties.
	 *
	 * @since 1.0
	 *
	 * @return array
	 */
	public function settings_to_chartjs_options_map(): array {
		/**
		 * Modifies a map of feed settings fields to Chart.js options.
		 *
		 * @filter `gk/gravitycharts/feed/settings-to-chart-library-options-map`
		 *
		 * @since  1.0
		 *
		 * @param array $fields
		 */
		return apply_filters(
			'gk/gravitycharts/feed/settings-to-chart-library-options-map',
			[
				'indexAxis'                    => [ 'indexAxis' ],
				'tension'                      => [ 'tension' ],
				'cutout'                       => [ 'cutout' ],
				'offset'                       => [ 'offset' ],
				'scales.r.angleLines.display'  => [ 'angleLinesDisplay' ],
				'scales.x.grid.display'        => [ 'xGridDisplay' ],
				'scales.y.grid.display'        => [ 'yGridDisplay' ],
				'scales.r.grid.display'        => [ 'rGridDisplay' ],
				'scales.r.ticks.display'       => [ 'ticksDisplay' ],
				'scales.r.pointLabels.display' => [ 'pointLabelsDisplay' ],
				'pointRadius'                  => [ 'pointRadius' ],
				'pointHoverRadius'             => [ 'pointRadius' ],
				'pointStyle'                   => [ 'pointStyle' ],
				'borderColor'                  => [ 'borderColor', 'borderColors' ],
				'borderWidth'                  => [ 'borderWidth' ],
				'backgroundColor'              => [ 'backgroundColor', 'backgroundColors' ],
				'plugins.legend.display'       => [ 'legendDisplay' ],
				'plugins.legend.position'      => [ 'legendPosition' ],
				'plugins.legend.align'         => [ 'legendAlign' ],
				'plugins.title.display'        => [ 'titleDisplay' ],
				'plugins.title.text'           => [ 'titleText' ],
				'plugins.title.font.size'      => [ 'titleSize' ],
			]
		);
	}

	/**
	 * Returns an array of default options to use with Chart.js.
	 *
	 * @since 1.0
	 *
	 * @return array
	 */
	public function default_chartjs_options(): array {
		/**
		 * Modifies default Chart.js options.
		 *
		 * @filter `gk/gravitycharts/chart-library-options`
		 *
		 * @since  1.0
		 *
		 * @param array  $fields  Array of default chart settings.
		 * @param string $library Name of the library being used.
		 */
		return apply_filters(
			'gk/gravitycharts/chart-library-options',
			[
				'plugins'               => [
					'legend'  => [
						'labels' => [
							'boxHeight' => 20,
						],
					],
					'tooltip' => [
						'displayColors' => false,
					],
					'title'   => [
						'display'  => false,
						'text'     => '',
						'fullSize' => true,
						'font'     => [
							'size' => 22,
						],
					],
				],
				'layout'                => [
					'padding' => 20,
				],
				'animation'             => false,
				'borderAlign'           => 'inner',
				'borderJoinStyle'       => 'round',
				'spanGaps'              => true,
				'fill'                  => true,
				'pointHitRadius'        => 0,
				'pointBorderWidth'      => 0,
				'pointHoverBorderWidth' => 0,
			],
			'chart.js'
		);
	}

	/**
	 * Cleans up field select choices before rendering.
	 *
	 * @since 1.0
	 *
	 * @param Fields\Base $field The field data.
	 * @param bool        $echo  Echo the output to the screen.
	 *
	 * @return string
	 */
	public function settings_field_select( $field, $echo = true ): ?string {
		$choices = [];

		foreach ( $field['choices'] as $choice ) {
			// Remove sub-fields, like checkboxes. These have decimal place value, e.g. "4.2".
			if ( false !== strpos( $choice['value'], '.' ) ) {
				continue;
			}

			/**
			 * Renders the color field type.
			 *
			 * @see GFAddon::get_field_map_choices()
			 */
			$selected_label = ' (' . esc_html__( 'Selected', 'gk-gravitycharts' ) . ')';
			$choice_label   = preg_replace( '/' . preg_quote( $selected_label, '/' ) . '$/m', '', $choice['label'] );

			$choices[] = [
				'value' => $choice['value'],
				'label' => $choice_label,
			];
		}

		$field['choices'] = $choices;

		// Get markup.
		$html = $field->prepare_markup();

		if ( $echo ) {
			echo wp_kses_post( $html );
		}

		return $html;
	}

	/**
	 * Renders the colors field type.
	 *
	 * @since 1.0
	 *
	 * @return array
	 */
	public function get_palette_choices(): array {
		$color_palettes = self::get_color_palettes();

		$choices = [];

		foreach ( $color_palettes as $name => $data ) {

			// Ensure 9 items in the palette.
			$border_colors = array_pad( $data['borderColor'], 9, null );

			$grid_css = '<style>';
			foreach ( $border_colors as $index => $color ) {
				$grid_css .= '.' . $name . '-color-fill-' . ( $index + 1 );
				$grid_css .= empty( $color ) ? '{display:none;}' : '{background-color: ' . $color . ';}';
			}
			$grid_css .= '</style>';

			$grid_title = rgar( $data, 'label', $name );
			$grid_title = esc_html( $grid_title );

			// Support RTL by showing tiles in reverse order.
			$order = is_rtl() ? [ 3, 2, 1, 6, 5, 4, 9, 8, 7 ] : [ 1, 2, 3, 4, 5, 6, 7, 8, 9 ];

			$tooltip = gform_tooltip( 'gravitycharts_palette_' . $name );

			$grid = <<<EOD
<span class="gravitycharts-palette-grid">
	<!-- <svg> This tells Gravity Forms to display this output as a choice icon. See GFCommon::get_icon_markup() -->
	$grid_css
	<span class="$name-color-fill-$order[0]"></span>
	<span class="$name-color-fill-$order[1]"></span>
	<span class="$name-color-fill-$order[2]"></span>
	<span class="$name-color-fill-$order[3]"></span>
	<span class="$name-color-fill-$order[4]"></span>
	<span class="$name-color-fill-$order[5]"></span>
	<span class="$name-color-fill-$order[6]"></span>
	<span class="$name-color-fill-$order[7]"></span>
	<span class="$name-color-fill-$order[8]"></span>
</span>
$tooltip
EOD;

			$choices[] = [
				'value' => $name,
				'label' => $grid_title,
				'icon'  => $grid,
			];
		}

		return $choices;
	}

	/**
	 * Renders the chart preview.
	 *
	 * @since 1.0
	 */
	public function settings_chart_preview() {
		?>
		<div id='gravitycharts-tab-wrapper'>
			<button class='button gravitycharts-settings-toggle gravitycharts-settings-toggle-closed' id='gravitycharts-button-colors' aria-controls='gform-settings-section-chart-colors'><span class='dashicons dashicons-art'></span> <?php esc_html_e( 'Colors', 'gk-gravitycharts' ); ?><span class='dashicons gravitycharts-settings-toggle-arrow'></span></button>
			<button class='button gravitycharts-settings-toggle gravitycharts-settings-toggle-closed' id='gravitycharts-button-design' aria-controls='gform-settings-section-chart-styles'><span class="dashicons dashicons-admin-appearance"></span> <?php esc_html_e( 'Design', 'gk-gravitycharts' ); ?> <span class='dashicons gravitycharts-settings-toggle-arrow'></span></button>
			<button class='button gravitycharts-settings-toggle gravitycharts-settings-toggle-closed' id='gravitycharts-button-title' aria-controls='gform-settings-section-legend'><span class='dashicons dashicons-editor-textcolor'></span> <?php esc_html_e( 'Title & Legend', 'gk-gravitycharts' ); ?> <span class='dashicons dashicons-arrow-down-alt2 gravitycharts-settings-toggle-arrow'></span></button>
			<button class='button gravitycharts-settings-toggle gravitycharts-settings-toggle-closed' id='gravitycharts-button-axis' aria-controls='gform-settings-section-axis'><span class='dashicons dashicons-grid-view'></span> <?php esc_html_e( 'Axis', 'gk-gravitycharts' ); ?> <span class='dashicons dashicons-arrow-down-alt2 gravitycharts-settings-toggle-arrow'></span></button>
		</div>
		<div id="gravitycharts-no-data-available" class="gravitycharts-notice alert warning notice-warning">
			<p>
				<?php esc_html_e( 'There are currently no entries for selected data source. The chart preview below is based on sample data.', 'gk-gravitycharts' ); ?>
			</p>
		</div>
		<canvas id="gravitycharts-preview"></canvas>
		<?php
	}

	/**
	 * Adds UI assets to GF's "no conflict" list.
	 *
	 * @since 1.0
	 *
	 * @param array $assets The current allowed assets.
	 *
	 * @return array
	 */
	private function allow_ui_assets( array $assets ): array {
		$assets[] = $this->query_filters::ASSETS_HANDLE;

		return $assets;
	}

	/**
	 * Configures translations for UI assets.
	 *
	 * @since 1.0
	 */
	private function configure_ui_translations() {
		if ( function_exists( 'wp_set_script_translations' ) ) {
			wp_set_script_translations( 'gravitycharts_feed_settings', 'gk-gravitycharts', plugin_dir_path( GK_GRAVITYCHARTS_PLUGIN_FILE ) . 'languages' );
		}
	}

	/***
	 * Renders the save button for settings pages.
	 *
	 * @since 1.0
	 *
	 * @param array $field Field array containing the configuration options of this field.
	 * @param bool  $echo  True to echo the output to the screen; false to simply return the contents as a string.
	 *
	 * @return string The HTML
	 */
	public function settings_submit( array $field, bool $echo = true ): string {
		$field['type'] = ( isset( $field['type'] ) && in_array(
			$field['type'],
			[
				'submit',
				'reset',
				'button',
			],
			true
		) ) ? $field['type'] : 'submit';

		$attributes    = $this->get_field_attributes( $field );
		$default_value = rgar( $field, 'value' ) ? rgar( $field, 'value' ) : rgar( $field, 'default_value' );
		$value         = $this->get_setting( $field['name'], $default_value );

		$attributes['class'] = isset( $field['class'] ) ? esc_attr( $field['class'] ) : $attributes['class'];

		$html       = ! empty( $field['html_before'] ) ? $field['html_before'] : '';
		$html_after = ! empty( $field['html_after'] ) ? $field['html_after'] : '';

		if ( ! rgar( $field, 'value' ) ) {
			$field['value'] = esc_html__( 'Update Settings', 'gk-gravitycharts' );
		}

		$attributes = $this->get_field_attributes( $field );

		unset( $attributes['html_before'], $attributes['html_after'], $attributes['tooltip'] );

		$html .= '<input
                    type="' . esc_attr( $field['type'] ) . '"
                    name="' . esc_attr( $field['name'] ) . '"
                    value="' . esc_attr( $value ) . '" ' .
				 implode( ' ', $attributes ) .
				 ' />';

		$html .= $html_after;
		$html .= wp_nonce_field( $this->_slug, '_wpnonce', true, false );

		if ( $echo ) {
			// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo $html;
		}

		return $html;
	}

	/**
	 * Don't show the uninstall section on the settings page.
	 *
	 * @return null
	 */
	public function render_uninstall() {
		return null;
	}

	/**
	 * Adds GravityCharts tooltips to the Gravity Forms tooltips array.
	 *
	 * @since 1.3
	 *
	 * @param array $tooltips The current tooltips.
	 *
	 * @return array
	 */
	public function add_tooltips( $tooltips ) {

		// translators: %s is replaced by a comma-separated list of fields that are supported by GravityCharts.
		$tooltips['gravitycharts_supported_fields'] = '<h6>' . esc_html__( 'Supported Field Types', 'gk-gravitycharts' ) . '</h6>' . sprintf( esc_html__( 'The following field types are currently supported: %s', 'gk-gravitycharts' ), implode( ', ', [ 'checkbox', 'select', 'radio', 'multiselect', 'likert', 'rank', 'rating', 'survey', 'quiz' ] ) );

		return $tooltips;
	}

	/**
	 * Determines if we're on the feed edit page.
	 *
	 * @since 1.3
	 *
	 * @return bool
	 */
	public function is_feed_edit_page() {
		if ( $this->is_gravityforms_supported( '2.5-beta' ) ) {
			return parent::is_feed_edit_page();
		}

		return 'gf_edit_forms' === rgget( 'page' ) && $this->get_slug() === rgget( 'subview' ) && array_key_exists( 'fid', $_GET ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
	}

	/**
	 * Determines if we're on the feed list page.
	 *
	 * @since 1.3
	 *
	 * @return bool
	 */
	public function is_feed_list_page() {
		return ! isset( $_GET['fid'] ) && ( 'gf_edit_forms' === rgget( 'page' ) && $this->get_slug() === rgget( 'subview' ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
	}

	/**
	 * Conditionally displays Help Scout beacon on certain pages.
	 *
	 * @since 1.3
	 *
	 * @param bool $display Whether to display the beacon.
	 *
	 * @return bool
	 */
	public function maybe_display_helpscout_beacon( $display ) {
		if ( $display ) {
			return true;
		}

		return $this->is_feed_edit_page() || $this->is_feed_list_page();
	}
}
