<?php
/**
 * Settings class for License settings
 * @since 1.0
 */
// Exit if accessed directly
if (!defined('ABSPATH')) {
   exit;
}

/**
 * Gsheet_License_Settings
 * @since 1.0
 */
class Gsheet_License_Settings {

   // GS Connector license key option name
   protected $gfgs_license_key_option_name = "gfgs_license_key";
   // GS Connector license status option name
   protected $gfgs_license_status_option_name = "gfgs_license_status";

   /**
    * Set things up.
    * @since 1.0
    */
   public function __construct() {
      add_action('admin_init', array($this, 'init_settings'));
   }

   // White list our options using the Settings API
   public function init_settings() {
      register_setting('gfgs-settings-license', $this->gfgs_license_key_option_name, array($this, 'validate_gfgs_license_key'));
   }

   /**
    * sanitize and validate the input (if required)
    * @since 1.0
    */
   public function validate_gfgs_license_key($license_key) {
      $license_key = sanitize_text_field($license_key);
      $gfgs_license_service = new GFGS_License_Service();

      if (isset($_POST['gfgs_license_deactivate'])) { // user is trying to deactivate the license
         $status = $gfgs_license_service->deactivate_license($license_key, $this->gfgs_license_status_option_name, GRAVITY_GOOGLESHEET_PRO_PRODUCT_NAME);

         if ($status == GFGS_License_Service::FAILED) {
            add_settings_error(
                    'gfgs-settings-license', 'gfgs-settings-license-gs-license-key', "GoogleSheetConnector - Gravityforms Pro " . __("License cannot be deactivated. Either the license key is invalid or the licensing server cannot be reached.", "gsheetconnector-gravityforms"), 'error'
            );
            // since there was an error, revert to the license key to the value from the DB
            $license_key = trim(get_option($this->gfgs_license_key_option_name));
         } else { // looks like we have a successful de-activation, so let's clear the license key
            $license_key = "";
         }
      } elseif (!empty($license_key)) { // user is trying to activate the license
         $status = $gfgs_license_service->activate_license($license_key, $this->gfgs_license_status_option_name, GRAVITY_GOOGLESHEET_PRO_PRODUCT_NAME);

         if ($status == GFGS_License_Service::INVALID) {
            add_settings_error(
                    'gfgs-settings-license', 'gfgs-settings-license-gs-license-key', "GoogleSheetConnector - Gravityforms Pro " . __("License cannot be activated. Either the license key is invalid or your activation limit is reached.", "gsheetconnector-gravityforms"), 'error'
            );
         }
      }

      return $license_key;
   }

   /*
    * generate the page
    *
    * @since 2.0
    */

   public function add_gfgs_setting_page() {
      $gfgs_license_key = get_option($this->gfgs_license_key_option_name);
      $gfgs_license_status = get_option($this->gfgs_license_status_option_name);
      ?>
      <form id="ggfs_settings_form" method="post" action="options.php">
         <?php
         // adds nonce and option_page fields for the settings page
         settings_fields('gfgs-settings-license');
         settings_errors();
         ?>
         <div class="wrap gs-form">
            <div class="card" id="googlesheet">
               <div class="select-info full-width">
                  <div>
                     <label class="settings-title" for="gfgs_license_key"><?php echo("GSheetConnector GravityForms Pro "); ?><?php _e('license key', "gsheetconnector-gravityforms"); ?>:</label>
                  </div>
                  <div class="gs-margin-top">
                     <input type="password" class="regular-text" name="<?php echo $this->gfgs_license_key_option_name; ?>" value="<?php echo $gfgs_license_key; ?>" />
                     <?php if ($gfgs_license_status !== false && $gfgs_license_status == 'valid') { ?>
                        <input type="submit" class="button-secondary" name="gfgs_license_deactivate" value="<?php _e('Deactivate License', "gsconnector"); ?>"/>
                     <?php } ?>
                  </div>
                  <br class="clear">
               </div>					
               <div class="select-info full-width">
                  <input type="submit" class="button button-primary button-large" name="gfgs_license_activate" value="<?php echo __("Save", "gsheetconnector-gravityforms"); ?>"/>
               </div>
               <br class="clear">
            </div>
         </div>
         <?php wp_nonce_field('gfgs_license_nonce', 'gfgs_license_nonce'); ?>
      </form>
      <?php
   }

}

$gs_license_settings = new Gsheet_License_Settings();
?>