<?php
/**
 * Register all actions and filters for the plugin
 *
 * @link       www.redefiningtheweb.com
 * @since      1.0.0
 *
 * @package    Rtwgftpdf_Gravity_Form_To_Pdf
 * @subpackage Rtwgftpdf_Gravity_Form_To_Pdf/includes
 */

/**
 * Register all actions and filters for the plugin.
 *
 * Maintain a list of all hooks that are registered throughout
 * the plugin, and register them with the WordPress API. Call the
 * run function to execute the list of actions and filters.
 *
 * @package    Rtwgftpdf_Gravity_Form_To_Pdf
 * @subpackage Rtwgftpdf_Gravity_Form_To_Pdf/includes
 * @author     RedefiningTheWeb <developer@redefiningtheweb.com>
 */

class Rtwgftpdf_Gravity_Form_To_Pdf_Loader {

	/**
	 * The array of actions registered with WordPress.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $rtwgftpdf_actions    The actions registered with WordPress to fire when the plugin loads.
	 */

	protected $rtwgftpdf_actions;

	/**
	 * The array of filters registered with WordPress.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $rtwgftpdf_filters    The filters registered with WordPress to fire when the plugin loads.
	 */

	protected $rtwgftpdf_filters;

	/**
	 * Initialize the collections used to maintain the actions and filters.
	 *
	 * @since    1.0.0
	 */

	public function __construct() 
	{
		$this->rtwgftpdf_actions = array();
		$this->rtwgftpdf_filters = array();
	}

	/**
	 * Add a new action to the collection to be registered with WordPress.
	 *
	 * @since    1.0.0
	 * @param    string               $rtwgftpdf_hook             The name of the WordPress action that is being registered.
	 * @param    object               $rtwgftpdf_component        A reference to the instance of the object on which the action is defined.
	 * @param    string               $rtwgftpdf_callback         The name of the function definition on the $rtwgftpdf_component.
	 * @param    int                  $rtwgftpdf_priority         Optional. The priority at which the function should be fired. Default is 10.
	 * @param    int                  $rtwgftpdf_accepted_args    Optional. The number of arguments that should be passed to the $rtwgftpdf_callback. Default is 1.
	 */

	public function rtwgftpdf_add_action( $rtwgftpdf_hook, $rtwgftpdf_component, $rtwgftpdf_callback, $rtwgftpdf_priority = 10, $rtwgftpdf_accepted_args = 1 ) 
	{
		$this->rtwgftpdf_actions = $this->rtwgftpdf_add( $this->rtwgftpdf_actions, $rtwgftpdf_hook, $rtwgftpdf_component, $rtwgftpdf_callback, $rtwgftpdf_priority, $rtwgftpdf_accepted_args );
	}

	/**
	 * Add a new filter to the collection to be registered with WordPress.
	 *
	 * @since    1.0.0
	 * @param    string               $rtwgftpdf_hook             The name of the WordPress filter that is being registered.
	 * @param    object               $rtwgftpdf_component        A reference to the instance of the object on which the filter is defined.
	 * @param    string               $rtwgftpdf_callback         The name of the function definition on the $rtwgftpdf_component.
	 * @param    int                  $rtwgftpdf_priority         Optional. The priority at which the function should be fired. Default is 10.
	 * @param    int                  $rtwgftpdf_accepted_args    Optional. The number of arguments that should be passed to the $rtwgftpdf_callback. Default is 1
	 */

	public function rtwgftpdf_add_filter( $rtwgftpdf_hook, $rtwgftpdf_component, $rtwgftpdf_callback, $rtwgftpdf_priority = 10, $rtwgftpdf_accepted_args = 1 ) 
	{
		$this->rtwgftpdf_filters = $this->rtwgftpdf_add( $this->rtwgftpdf_filters, $rtwgftpdf_hook, $rtwgftpdf_component, $rtwgftpdf_callback, $rtwgftpdf_priority, $rtwgftpdf_accepted_args );
	}

	/**
	 * A utility function that is used to register the actions and hooks into a single
	 * collection.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @param    array                $rtwgftpdf_hooks            The collection of hooks that is being registered (that is, actions or filters).
	 * @param    string               $rtwgftpdf_hook             The name of the WordPress filter that is being registered.
	 * @param    object               $rtwgftpdf_component        A reference to the instance of the object on which the filter is defined.
	 * @param    string               $rtwgftpdf_callback         The name of the function definition on the $component.
	 * @param    int                  $rtwgftpdf_priority         The priority at which the function should be fired.
	 * @param    int                  $rtwgftpdf_accepted_args    The number of arguments that should be passed to the $rtwgftpdf_callback.
	 * @return   array                                  The collection of actions and filters registered with WordPress.
	 */

	private function rtwgftpdf_add( $rtwgftpdf_hooks, $rtwgftpdf_hook, $rtwgftpdf_component, $rtwgftpdf_callback, $rtwgftpdf_priority, $rtwgftpdf_accepted_args ) 
	{
		$rtwgftpdf_hooks[] = array(
			'hook'          => $rtwgftpdf_hook,
			'component'     => $rtwgftpdf_component,
			'callback'      => $rtwgftpdf_callback,
			'priority'      => $rtwgftpdf_priority,
			'accepted_args' => $rtwgftpdf_accepted_args
		);
		return $rtwgftpdf_hooks;
	}

	/**
	 * Register the filters and actions with WordPress.
	 *
	 * @since    1.0.0
	 */

	public function rtwgftpdf_run() 
	{
		foreach ( $this->rtwgftpdf_filters as $rtwgftpdf_hook ) 
		{
			add_filter( $rtwgftpdf_hook['hook'], array( $rtwgftpdf_hook['component'], $rtwgftpdf_hook['callback'] ), $rtwgftpdf_hook['priority'], $rtwgftpdf_hook['accepted_args'] );
		}

		foreach ( $this->rtwgftpdf_actions as $rtwgftpdf_hook ) 
		{
			add_action( $rtwgftpdf_hook['hook'], array( $rtwgftpdf_hook['component'], $rtwgftpdf_hook['callback'] ), $rtwgftpdf_hook['priority'], $rtwgftpdf_hook['accepted_args'] );
		}
	}
}